import typing

from aws_cdk.aws_ecr import Repository, LifecycleRule, TagStatus
from aws_cdk.core import Construct, RemovalPolicy


class ContainerRepository(Construct):
    def __init__(self, scope: Construct, id: str, *, repository_name: typing.Optional[str] = None):
        super().__init__(scope, id)
        remove_untagged_images_rule = LifecycleRule(description='Remove untagged images', max_image_count=1,
                                                    rule_priority=1,
                                                    tag_status=TagStatus.UNTAGGED)
        Repository(scope, f'${id}-repository', image_scan_on_push=True, repository_name=repository_name,
                   removal_policy=RemovalPolicy.DESTROY,
                   lifecycle_rules=[remove_untagged_images_rule])



import typing

from aws_cdk.aws_ec2 import IConnectable, Port
from aws_cdk.aws_ecr import Repository
from aws_cdk.aws_ecs import FargateTaskDefinition, ICluster, ContainerImage, LogDriver, FargateService, PortMapping, \
    Protocol
from aws_cdk.aws_logs import RetentionDays
from aws_cdk.core import Construct
from jsii import Number


class StandaloneFargateService(Construct):
    def __init__(self, scope, id: str, *, service_name: str, cluster: ICluster, task_cpu: Number,
                 task_mem_limit_mib: Number, ecr_repo_name: str, image_tag: str, desired_container_count: int,
                 environment: typing.Optional[typing.Mapping[str, str]] = None):
        super().__init__(scope, id)

        task_definition = FargateTaskDefinition(scope, f'{service_name}-task-definition', cpu=task_cpu,
                                                memory_limit_mib=task_mem_limit_mib)
        image_repo = Repository.from_repository_name(scope, f'{service_name}-repository',
                                                     repository_name=ecr_repo_name)
        container_definition = task_definition.add_container(
            f'{service_name}-container',
            image=ContainerImage.from_ecr_repository(image_repo, tag=image_tag),
            logging=LogDriver.aws_logs(stream_prefix=service_name, log_retention=RetentionDays.ONE_WEEK),
            memory_limit_mib=task_mem_limit_mib, cpu=task_cpu, environment=environment)
        container_definition.add_port_mappings(PortMapping(container_port=80, host_port=80, protocol=Protocol.TCP))
        self.service = FargateService(scope, f'{service_name}-service', task_definition=task_definition,
                                      cluster=cluster,
                                      desired_count=desired_container_count, min_healthy_percent=50,
                                      max_healthy_percent=100,
                                      service_name=service_name)

    @property
    def service_instance(self):
        return self.service

    def allow_connection_from(self, other: IConnectable, port: Port = None):
        connection_port = port or Port.tcp(80)
        return self.service.connections.allow_from(other, connection_port)



import typing

from aws_cdk.aws_kms import Key, IKey
from aws_cdk.core import Construct, RemovalPolicy
from aws_cdk.aws_s3 import Bucket, BucketAccessControl, BlockPublicAccess, BucketEncryption


class GICBucket(Construct):
    def __init__(self, scope: Construct, id: str, bucket_name: str, *, encryption_key: typing.Optional[IKey] = None):
        super().__init__(scope, id)
        if not encryption_key:
            encryption_key = Key(self, f'{bucket_name}-s3-key',
                                 enable_key_rotation=True,
                                 description='Key for encryption at rest for S3',
                                 alias=f'{bucket_name}/s3')
        access_log_bucket = Bucket(self, f'{id}-access-logs',
                                   block_public_access=BlockPublicAccess.BLOCK_ALL,
                                   bucket_name=f'{bucket_name}-access-logs',
                                   versioned=False,
                                   removal_policy=RemovalPolicy.DESTROY)
        self.bucket = Bucket(self, bucket_name,
                             access_control=BucketAccessControl.PRIVATE,
                             block_public_access=BlockPublicAccess.BLOCK_ALL,
                             bucket_name=bucket_name,
                             encryption=BucketEncryption.KMS,
                             encryption_key=encryption_key,
                             versioned=True,
                             removal_policy=RemovalPolicy.DESTROY,
                             server_access_logs_bucket=access_log_bucket,
                             server_access_logs_prefix=bucket_name
                             )

    @property
    def bucket_name(self):
        return self.bucket.bucket_name



import typing

from aws_cdk.aws_ec2 import InstanceType, InstanceSize, InstanceClass, IVpc, Port, IConnectable
from aws_cdk.aws_kms import IKey
from aws_cdk.aws_rds import DatabaseInstance, DatabaseInstanceEngine, StorageType, ParameterGroup, OptionGroup
from aws_cdk.core import Construct, Duration

INSTANCE_CLASS_MEMEORY_MULTIPLIER_MAPPING = {
    InstanceClass.STANDARD4: 1,
    InstanceClass.STANDARD5: 1,
    InstanceClass.BURSTABLE2: 1,
    InstanceClass.BURSTABLE3: 1,
    InstanceClass.MEMORY4: 2,
    InstanceClass.MEMORY5: 2,
}

INSTANCE_SIZE_TO_MEMORY_MAPPING = {
    InstanceSize.SMALL: 2,
    InstanceSize.MEDIUM: 4,
    InstanceSize.LARGE: 8,
    InstanceSize.XLARGE: 16,
    InstanceSize.XLARGE2: 32,
    InstanceSize.XLARGE4: 64,
    InstanceSize.XLARGE10: 160,
    InstanceSize.XLARGE12: 192,
    InstanceSize.XLARGE16: 256,
    InstanceSize.XLARGE24: 384,
}


class Postgres(Construct):
    def __init__(self, scope: Construct, id: str, *, master_username: str, vpc: IVpc, database_name: str,
                 instance_class: InstanceClass, instance_size: InstanceSize, instance_identifier: str = None,
                 version: str = '10.11', encryption_key: typing.Optional[IKey] = None) -> None:
        super().__init__(scope, id)
        db_memory_gb = INSTANCE_SIZE_TO_MEMORY_MAPPING[instance_size] * INSTANCE_CLASS_MEMEORY_MULTIPLIER_MAPPING[
            instance_class]
        if db_memory_gb <= 4:
            work_mem = '1024'
        elif db_memory_gb <= 8:
            work_mem = '2048'
        else:
            work_mem = '3072'
        parameters = {
            # 'archive_timeout': '900',
            'authentication_timeout': '60',
            'autovacuum': 'on',
            'autovacuum_analyze_threshold': '50',
            'autovacuum_freeze_max_age': '200000000',
            'autovacuum_max_workers': '2',
            'autovacuum_multixact_freeze_max_age': '400000000',
            'autovacuum_naptime': '60',
            'autovacuum_vacuum_scale_factor': '0.2',
            'autovacuum_vacuum_threshold': '50',
            'autovacuum_work_mem': '-1' if db_memory_gb <= 8 else f'{512 * 1024}',
            'bgwriter_delay': '150',
            'checkpoint_completion_target': '0.7',
            'checkpoint_timeout': '900',
            'effective_cache_size': f'{int(db_memory_gb * 0.5 * 1024 * 1024 / 8)}',
            'lc_messages': 'en_US.UTF-8',
            'lc_monetary': 'en_US.UTF-8',
            'lc_numeric': 'en_US.UTF-8',
            'lc_time': 'en_US.UTF-8',
            # 'listen_addresses': '*',
            'log_autovacuum_min_duration': '0',
            'log_checkpoints': 'on',
            'log_connections': 'on',
            'log_destination': 'stderr',
            'log_disconnections': 'on',
            'log_duration': '0',
            'log_error_verbosity': 'verbose',
            'log_executor_stats': '0',
            # 'log_file_mode': '0600 or above',
            'log_hostname': 'off',
            # 'log_line_prefix': 'at %t - from Application [%a] on server [%r] using user [%u] for dbname [%d] process id [%p] returned SQLSTATE [%e]:',
            'log_lock_waits': 'on',
            'log_min_duration_statement': '15000',
            'log_min_error_statement': 'error',
            'log_min_messages': 'warning',
            'log_rotation_age': '1440',
            'log_rotation_size': '10240',
            'log_statement': 'ddl',
            'log_statement_stats': '0',
            'log_temp_files': '-1',
            # 'log_truncate_on_rotation': 'off',
            'maintenance_work_mem': '262144',
            'max_files_per_process': '1000',
            'max_locks_per_transaction': '64',
            # 'max_parallel_maintenance_workers': '8',
            'max_parallel_workers': '8',
            'max_parallel_workers_per_gather': '2',
            'max_pred_locks_per_page': '2',
            'max_pred_locks_per_relation': '2',
            # 'max_replication_slots': 'match with the no.of replication servers available ',
            # 'max_wal_senders': 'match with the no.of replication servers available ',
            'rds.force_admin_logging_level': 'Warning',
            'rds.force_autovacuum_logging_level': 'error',
            'rds.log_retention_period': '10080',
            'rds.rds_superuser_reserved_connections': '3',
            'rds.restrict_password_commands': '1',
            'search_path': '$user,public',
            'shared_buffers': f'{int(0.1 * 1024 * 1024 * db_memory_gb / 8)}' if db_memory_gb <= 8 else f'{int(.25 * 1024 * 1024 * db_memory_gb / 8)}',
            'shared_preload_libraries': 'pg_stat_statements,pgaudit',
            # 'superuser_reserved_connections': '3',
            'track_functions': 'all',
            'wal_buffers': str(int(128 * 1024 / 8)) if db_memory_gb <= 8 else str(int(512 * 1024 / 8)),
            'wal_keep_segments': '32',
            'work_mem': work_mem
        }
        parameter_group = ParameterGroup(scope, 'rds-parameter-group', family='postgres10', parameters=parameters)
        option_group = OptionGroup(scope, 'rds-option-group', engine=DatabaseInstanceEngine.POSTGRES,
                                   major_engine_version='10', configurations=[])
        self.instance = DatabaseInstance(scope, "db-instance",
                                         engine=DatabaseInstanceEngine.POSTGRES,
                                         engine_version=version,
                                         instance_class=InstanceType.of(instance_class, instance_size),
                                         multi_az=True,
                                         storage_type=StorageType.IO1,
                                         allocated_storage=120,
                                         master_username=master_username,
                                         kms_key=encryption_key,
                                         vpc=vpc,
                                         database_name=database_name,
                                         storage_encrypted=True,
                                         backup_retention=Duration.days(7),
                                         monitoring_interval=Duration.seconds(60),
                                         enable_performance_insights=True,
                                         auto_minor_version_upgrade=True,
                                         parameter_group=parameter_group,
                                         option_group=option_group,
                                         deletion_protection=False,
                                         instance_identifier=instance_identifier
                                         )

    @property
    def endpoint(self):
        return self.instance.instance_endpoint.hostname

    def allow_connection_from(self, other: IConnectable, port: Port = None):
        connection_port = port or Port.tcp(5432)
        return self.instance.connections.allow_from(other, connection_port)



from aws_cdk.aws_ec2 import SecurityGroup, Port, IVpc, SubnetType, IConnectable
from aws_cdk.aws_elasticache import CfnReplicationGroup, CfnSubnetGroup
from aws_cdk.core import Construct


class Redis(Construct):
    def __init__(self, scope: Construct, id: str, *, vpc: IVpc, node_count: int,
                 cache_node_type: str):
        super().__init__(scope, id)
        self.security_group = SecurityGroup(scope, 'redis-security-group', allow_all_outbound=True, vpc=vpc)
        private_subnets = vpc.select_subnets(subnet_type=SubnetType.PRIVATE)
        subnet_group = CfnSubnetGroup(scope, 'redis-subnet-group', description='redis-subnet-groups',
                                      subnet_ids=private_subnets.subnet_ids,
                                      cache_subnet_group_name=f'subnet-group-{id}')
        self.replication_group = CfnReplicationGroup(scope, 'redis-cluster', engine='redis',
                                                     automatic_failover_enabled=True,
                                                     auto_minor_version_upgrade=False, cache_node_type=cache_node_type,
                                                     cache_subnet_group_name=subnet_group.cache_subnet_group_name,
                                                     replicas_per_node_group=node_count,
                                                     security_group_ids=[self.security_group.security_group_id],
                                                     replication_group_description=f'Multi AZ with {node_count} nodes',
                                                     at_rest_encryption_enabled=True,
                                                     transit_encryption_enabled=True,
                                                     )
        self.replication_group.add_depends_on(subnet_group)

    @property
    def primary_address(self):
        return self.replication_group.attr_primary_end_point_address

    @property
    def primary_port(self):
        return self.replication_group.attr_primary_end_point_port

    def allow_connection_from(self, other: IConnectable, port: Port = None):
        connection_port = port or Port.tcp(6379)
        return self.security_group.connections.allow_from(other, connection_port)



import setuptools

with open("README.md") as fp:
    long_description = fp.read()

setuptools.setup(
    name="labs_cdk",
    version="0.0.11",
    author="Vaibhav Bansal",
    author_email="vaibhavbansal@gic.com.sg",
    description="GIC Labs AWS CDK constructs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/GICLabs/labs-cdk",
    packages=setuptools.find_packages(),
    install_requires=[
        "attrs==19.3.0",
        "aws-cdk.assets==1.41.0",
        "aws-cdk.aws-apigateway==1.41.0",
        "aws-cdk.aws-applicationautoscaling==1.41.0",
        "aws-cdk.aws-autoscaling==1.41.0",
        "aws-cdk.aws-autoscaling-common==1.41.0",
        "aws-cdk.aws-autoscaling-hooktargets==1.41.0",
        "aws-cdk.aws-batch==1.41.0",
        "aws-cdk.aws-certificatemanager==1.41.0",
        "aws-cdk.aws-cloudformation==1.41.0",
        "aws-cdk.aws-cloudfront==1.41.0",
        "aws-cdk.aws-cloudwatch==1.41.0",
        "aws-cdk.aws-codebuild==1.41.0",
        "aws-cdk.aws-codecommit==1.41.0",
        "aws-cdk.aws-codepipeline==1.41.0",
        "aws-cdk.aws-cognito==1.41.0",
        "aws-cdk.aws-ec2==1.41.0",
        "aws-cdk.aws-ecr==1.41.0",
        "aws-cdk.aws-ecr-assets==1.41.0",
        "aws-cdk.aws-ecs==1.41.0",
        "aws-cdk.aws-ecs-patterns==1.41.0",
        "aws-cdk.aws-elasticache==1.41.0",
        "aws-cdk.aws-elasticloadbalancing==1.41.0",
        "aws-cdk.aws-elasticloadbalancingv2==1.41.0",
        "aws-cdk.aws-events==1.41.0",
        "aws-cdk.aws-events-targets==1.41.0",
        "aws-cdk.aws-iam==1.41.0",
        "aws-cdk.aws-kms==1.41.0",
        "aws-cdk.aws-lambda==1.41.0",
        "aws-cdk.aws-logs==1.41.0",
        "aws-cdk.aws-rds==1.41.0",
        "aws-cdk.aws-route53==1.41.0",
        "aws-cdk.aws-route53-targets==1.41.0",
        "aws-cdk.aws-s3==1.41.0",
        "aws-cdk.aws-s3-assets==1.41.0",
        "aws-cdk.aws-sam==1.41.0",
        "aws-cdk.aws-secretsmanager==1.41.0",
        "aws-cdk.aws-servicediscovery==1.41.0",
        "aws-cdk.aws-sns==1.41.0",
        "aws-cdk.aws-sns-subscriptions==1.41.0",
        "aws-cdk.aws-sqs==1.41.0",
        "aws-cdk.aws-ssm==1.41.0",
        "aws-cdk.aws-stepfunctions==1.41.0",
        "aws-cdk.cloud-assembly-schema==1.41.0",
        "aws-cdk.core==1.41.0",
        "aws-cdk.custom-resources==1.41.0",
        "aws-cdk.cx-api==1.41.0",
        "aws-cdk.region-info==1.41.0",
        "cattrs==1.0.0",
        "constructs==3.0.3",
        "jsii==1.5.0",
        "publication==0.0.3",
        "python-dateutil==2.8.1",
        "six==1.14.0",
        "typing-extensions==3.7.4.2"
    ],
    python_requires=">=3.6",
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)



import typing

from aws_cdk.aws_certificatemanager import ICertificate
from aws_cdk.aws_ecr import Repository
from aws_cdk.aws_ecs import ICluster, FargateTaskDefinition, ContainerImage, LogDriver, PortMapping, Protocol
from aws_cdk.aws_ecs_patterns import ApplicationLoadBalancedFargateService
from aws_cdk.aws_elasticloadbalancingv2 import ApplicationProtocol, IApplicationLoadBalancer
from aws_cdk.aws_logs import RetentionDays
from aws_cdk.aws_route53 import HostedZone
from aws_cdk.core import Construct
from jsii import Number


class AlbFargateService(Construct):
    def __init__(self, scope: Construct, id: str, *, service_name: str, cluster: ICluster, task_cpu: Number,
                 task_mem_limit_mib: Number, ecr_repo_name: str, image_tag: str, desired_container_count: int,
                 environment: typing.Optional[typing.Mapping[str, str]] = None,
                 load_balancer: typing.Optional[IApplicationLoadBalancer] = None,
                 certificate: typing.Optional[ICertificate] = None):
        super().__init__(scope, id)

        task_definition = FargateTaskDefinition(scope, f'{service_name}-task-definition', cpu=task_cpu,
                                                memory_limit_mib=task_mem_limit_mib)
        image_repo = Repository.from_repository_name(scope, f'{service_name}-repository',
                                                     repository_name=ecr_repo_name)
        container_definition = task_definition.add_container(
            f'{service_name}-container',
            image=ContainerImage.from_ecr_repository(image_repo, tag=image_tag),
            logging=LogDriver.aws_logs(stream_prefix=service_name, log_retention=RetentionDays.ONE_WEEK),
            memory_limit_mib=task_mem_limit_mib, cpu=task_cpu, environment=environment)
        container_definition.add_port_mappings(PortMapping(container_port=80, host_port=80, protocol=Protocol.TCP))
        self.service = ApplicationLoadBalancedFargateService(
            scope, f'{service_name}-service',
            cpu=desired_container_count * task_cpu,
            memory_limit_mib=desired_container_count * task_mem_limit_mib,
            task_definition=task_definition,
            cluster=cluster,
            desired_count=desired_container_count,
            min_healthy_percent=50,
            max_healthy_percent=100, listener_port=80,
            public_load_balancer=True,
            protocol=ApplicationProtocol.HTTPS,
            domain_name='giclabs.com',
            domain_zone=HostedZone.from_hosted_zone_attributes(scope, f'giclabs-zone-{id}',
                                                               hosted_zone_id='Z3U2I70U6FV8LX',
                                                               zone_name='giclabs.com'),
            service_name=service_name,
            load_balancer=load_balancer,
            certificate=certificate)

    @property
    def load_balancer(self):
        return self.service.load_balancer

    @property
    def service_instance(self):
        return self.service.service
